#include "Molecule.hpp"

namespace fragdock {
  std::ostream& operator<< (std::ostream& os, const Bond& bond) {
    os << "From:" << bond.atom_id1 << " To:" << bond.atom_id2;
    return os;
  }


  void Molecule::translate(const Vector3d &vec) {
    for(int i = 0; i < atoms.size(); i++) 
      atoms[i].translate(vec);
  }

  void Molecule::rotate(const Rotation &rot) {
    translate(-getCenter());
    for(int i = 0; i < atoms.size(); i++)
      atoms[i].rotate(rot);
  }

  void Molecule::rotate(fltype theta, fltype phi, fltype psi) {
    translate(-getCenter());
    for(int i = 0; i < atoms.size(); i++)
      atoms[i].rotate(theta, phi, psi);
  }

  void Molecule::axisRotate(const Vector3d &axis, fltype th) {
    for(int i = 0; i < atoms.size(); i++)
      atoms[i].axisRotate(axis, th);
  }

  void Molecule::axisRotate(const Vector3d& base, const Vector3d& axis, fltype th, const std::vector<int>& id_set) {
    Vector3d pos = getCenter();
    translate(-base);
    for(int i = 0; i < id_set.size(); i++)
      atoms[id_set[i]].axisRotate(axis, th);
    translate(pos - getCenter());
  }

  void Molecule::append(const Molecule &o) {
    for(int i = 0; i < o.size(); i++)
      append(o.getAtom(i));
  }

  void Molecule::append(const Atom &o) {
    atoms.push_back(o);
  }

  void Molecule::append(const Bond &bond) {
    bonds.push_back(bond);
  }

  Vector3d Molecule::getCenter() const {
    Vector3d center(0.0, 0.0, 0.0);
    for(int i = 0; i < atoms.size(); i++)
      center += atoms[i];
    return center/(fltype)atoms.size();
  }
  void Molecule::sort() {
    std::sort(atoms.begin(), atoms.end());    
  }
  fltype Molecule::calcRMSD(const Molecule& o) const {
    using namespace std;
    if(this->size() != o.size()) {
      cerr << "calcRMSD: atom size is differ!!" << endl;
    }
    fltype sum = 0.0;
    for(int i = 0; i < this->size(); i++) {
      sum += (this->getAtom(i) - o.getAtom(i)).norm();
    }
    sum /= this->size();
    return sqrt(sum);
  }
  
  std::ostream& operator<< (std::ostream& os, const Molecule& mol){
    std::vector<Atom> atoms = mol.atoms;
    std::vector<Bond> bonds = mol.bonds;

    for(int i=0; i<atoms.size(); i++){
      Atom atom = atoms[i];
      os << atom << std::endl;
    }

    for(int i=0; i<bonds.size(); i++){
      Bond bond = bonds[i];
      os << "BondId:" << i << " " << bond << std::endl;
    }
    return os;
  }
}
