#include <string>//stod
#include <iostream>
#include <fstream>
#include <vector>
#include <sstream> //string stream

#include <map>

#include <boost/algorithm/string/classification.hpp> // is_any_of
#include <boost/algorithm/string/split.hpp>
#include <boost/lexical_cast.hpp>

#include "ff_reader.hpp"
#include "utils.hpp"
#include "common.hpp" //typedef fltype

namespace docking{

std::map<std::string, AtomParam> GetAtomParam(const char* filename){
  std::ifstream itp_stream(filename);
  if(itp_stream.fail()){
    std::cerr << "failed" << std::endl;
    abort();
  }
  std::string itp_record;

  std::map<std::string, docking::AtomParam> paramMap;

  while(!itp_stream.eof()){
    std::getline(itp_stream, itp_record);
    
    if(itp_record.find("[ atomtypes ]") != -1){
      continue;
    }
    
    std::vector<std::string> result;
    itp_record = itp_record.substr(0, itp_record.find_first_of(';'));
    boost::algorithm::split(result, itp_record, boost::is_any_of("\t"));

    std::string atomType;
    fltype charge, sigma, epsilon;
    std::stringstream ss; //string to fltype

    if(result.size() > 6){ 
      atomType = result[0];

      charge  = boost::lexical_cast<double>(utils::trim_tail(result[3]));
      sigma   = boost::lexical_cast<double>(utils::trim_tail(result[5])) * 10; //convert from nanometer to angstrom
      epsilon = boost::lexical_cast<double>(utils::trim_tail(result[6]));

      paramMap[atomType] = (AtomParam){charge, sigma, epsilon};
      charge = sigma = epsilon = 1;
    }
  }

  return paramMap;
}
}; //namespace docking

/*
int main(void){
  std::map<std::string, docking::AtomParam> paramMap;
  const char* filename = "./nonbonded.itp";

  paramMap = docking::GetAtomParam(filename);
  for(std::map<std::string, docking::AtomParam>::iterator it = paramMap.begin(); it != paramMap.end(); ++it){
    std::cout << it->first << " " << it->second.charge << " " << it->second.sigma << " " << it->second.epsilon << std::endl;
  }
  
}
*/
