#include <iostream>
#include <fstream>

#include <boost/regex.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>

#include "paramAD_reader.hpp"
#include "utils.hpp"

namespace docking{

std::map<std::string, docking::AtomParamAD> GetAtomParamAD(const char* filename){
  std::ifstream dat_stream(filename);
  if(dat_stream.fail()){
    std::cerr << "failed" << std::endl;
    abort();
  }
  std::string line;

  std::map<std::string, docking::AtomParamAD> paramMap;

  while(!dat_stream.eof()){
    getline(dat_stream, line);

    line = utils::trim_head(line);
    line = line.substr(0, line.find_first_of("#"));
    line = utils::trim_tail(line);

    if(line.size() == 0) continue;
    
    static boost::regex express("atom_par");
    boost::smatch result;
    if(boost::regex_search(line, result, express)){
      docking::AtomParamAD atom_par_AD;
      atom_par_AD.atomNumber = paramMap.size();
      atom_par_AD.vdw_sigma = boost::lexical_cast<fltype>(utils::trim_head(line.substr(16,4)));
      atom_par_AD.vdw_epsilon = boost::lexical_cast<fltype>(utils::trim_head(line.substr(22,5)));
      atom_par_AD.vol = boost::lexical_cast<fltype>(utils::trim_head(line.substr(29,7)));
      atom_par_AD.solpar = boost::lexical_cast<fltype>(utils::trim_head(line.substr(38,8)));
      atom_par_AD.hb_sigma = boost::lexical_cast<fltype>(utils::trim_head(line.substr(48,3)));
      atom_par_AD.hb_epsilon = boost::lexical_cast<fltype>(utils::trim_head(line.substr(53,3)));
      int hb_type = boost::lexical_cast<int>(line.substr(58,1));
      atom_par_AD.hb_type = static_cast<docking::HydrogenBond::TypeEnum>(0x00000001 << hb_type);
      paramMap[utils::trim_tail(line.substr(9,2))] = atom_par_AD;
    }
  }


  return paramMap;
}
}; //namespace docking

