#include "pdbqt_reader.hpp"
#include "pdbqt_writer.hpp"

#include <iostream>
#include <fstream>
#include <iomanip> //setw, setprecision, etc....
#include <string>
#include <vector>
#include <cstdlib> //abort


namespace format{
void outputPDBQTFile(const char *filename, const format::PDBQTMolecule *pdbqt){
  using namespace std;
  
  std::ofstream fout(filename);
  if(fout.fail()){
    std::cerr << "ERROR:outputPDBQTFile: opening file failed:" << filename << std::endl;
    abort();
  }

  fout << "REMARK   4 XXXX COMPLIES WITH FORMAT V. 2.0" << std::endl;;
  for(std::vector<format::PDBQTAtom>::const_iterator atom=pdbqt->atoms.begin();
      atom!=pdbqt->atoms.end();
      atom++){
    fout << "ATOM  " << " "
	 << setw(4) << right << atom->id << " "
	 << setw(3) << left << atom->name << " "
	 << setw(3) << right << atom->residue_name << " "
	 << atom->chainID << " "
	 << setw(3) << right << atom->residueID << "     "
	 << setw(7) << right << fixed << setprecision(3) << atom->x << " "
	 << setw(7) << right << fixed << setprecision(3) << atom->y << " "
	 << setw(7) << right << fixed << setprecision(3) << atom->z << " "
	 << setw(5) << right << fixed << setprecision(2) << 0.00 << " "
	 << setw(5) << right << fixed << setprecision(2) << 0.00 << "   "
	 << setw(7) << right << fixed << setprecision(3) << atom->charge << " "
	 << setw(2) << left  << atom->element  << std::endl;
  }
  fout << "TER   " << " " 
       << setw(4) << right << (pdbqt->atoms.size()+1) << "      "
       << setw(3) << right << pdbqt->atoms.back().residue_name << " "
       << pdbqt->atoms.back().chainID << " "
       << setw(3) << right << pdbqt->atoms.back().residueID << std::endl;

  for(std::vector<format::PDBQTBond>::const_iterator bond=pdbqt->bonds.begin();
      bond!=pdbqt->bonds.end();
      bond++){
    fout << "CONECT" << " "
	 << setw(4) << right << bond->origin_atom_id << " "
	 << setw(4) << right << bond->target_atom_id << " "
	 << std::endl;
  }

  fout.close();
}
}//namespace format
