#include <map>
#include <vector>
#include "ff_reader.hpp"
#include "paramAD_reader.hpp"
#include "common.hpp"
//#include "pdb_reader.hpp"
#include "mol_reader.hpp"
#include "infile_reader.hpp"
#include "utils.hpp"
#include "energy_grid.hpp"

//below includings are from fragdock
#include "Molecule.hpp"
#include "EnergyGrid.hpp"

#ifndef DOCKING_H_
#define DOCKING_H_
namespace docking{

  struct DockBond{
    int bond_id;
    bool isRotatable;
    int origin_atom_id, origin_atom_index;
    int target_atom_id, target_atom_index;
    format::BondType::TypeEnum bond_type;
    std::string status_bits;
  };
  struct DockAtom{
    int atom_id;
    int atom_type_id;
    bool is_heavy_atom;
    fltype x,y,z;
    fltype vdw_sigma, vdw_epsilon;
    fltype charge;
    fltype volume, solpar;
    fltype hbond_sigma, hbond_epsilon;
    docking::HydrogenBond::TypeEnum hb_type;
    //std::vector<docking::DockBond> adjacent_bonds; //not use now.
    void rotate(const fltype &theta, const fltype &phi, const fltype &psi);
  };
  struct DockMolecule{
    std::vector<docking::DockAtom> atoms;
    std::vector<docking::DockBond> bonds;
  };



  struct RotInfo{
    fltype theta,phi,psi;

    bool operator<(const RotInfo &o) const{
      return theta!=o.theta ? theta<o.theta : (phi!=o.phi?phi<o.phi:psi<o.psi);
    }
    bool operator!=(const RotInfo &o) const{
      return theta!=o.theta || phi!=o.phi || psi!=o.psi;
    }
  };
  struct Pose{
    fragdock::Vector3d trinfo;
    RotInfo rotinfo;

    bool operator<(const Pose &o) const{
      bool ret = trinfo.abs() < o.trinfo.abs();
      bool equal = trinfo.abs() == o.trinfo.abs();
      return !equal ? ret : rotinfo<o.rotinfo;
    }
  };

  void MakeTranslations(const fragdock::Point3d<fltype> &center, 
			const fragdock::Point3d<fltype> &pitch, 
			const fragdock::Point3d<int> &num,
			std::vector<fragdock::Vector3d> &translations);

  void OmitTranslations(const docking::EnergyGrid &ignoreGrid, 
			std::vector<fragdock::Vector3d> &translations);

  void MakeRotations(const std::string &Filename, 
		     std::vector<docking::RotInfo> &rotations);

  int NumNearAtoms(const fragdock::Molecule& receptor, const fragdock::Vector3d& point, fltype threashold_dist);

  int GetNearAtoms(const std::vector<docking::DockAtom>* ratoms, 
		   const utils::Vec3D<fltype> *point,
		   fltype threasholdDist, 
		   std::vector<docking::DockAtom>* retAtoms);

  //newer version.
  void docking(const fragdock::Molecule& receptor,
	       const fragdock::Molecule& ligand,
	       const std::vector<fragdock::Vector3d>& translations,
	       const std::vector<docking::RotInfo>& rotangs,
	       const std::string& grid_folder,
	       std::vector< std::pair<fltype, Pose> >& conformations,
	       int num_of_outputs = 1);

}
#endif
